// Lis example: steady heat transfer

#include <iostream>
#include <cstdlib>
#include "lis.h"

double const D = 1.;		// Coefficient of heat transfer

int const ndiv = 20;
double const dx = 1. / ndiv;
double const Tw = 0., Te = 100.;

int main(int argc, char* argv[])
{
  lis_initialize(&argc, &argv);

  LIS_SOLVER solver;
  LIS_MATRIX A;
  LIS_VECTOR b, x;

  lis_solver_create(&solver);
  lis_matrix_create(0, &A);
  lis_vector_create(0, &b);
  lis_vector_create(0, &x);

  lis_matrix_set_size(A, 0, ndiv - 1);
  lis_vector_set_size(b, 0, ndiv - 1);
  lis_vector_set_size(x, 0, ndiv - 1);

  for (int i = 0; i < ndiv - 1; ++i) {
    // The coefficient matrix
    if (i == 0) {
      // A(i, i  ) = -2.*D/(dx*dx)
      lis_matrix_set_value(LIS_INS_VALUE, i, i  , -2.*D/(dx*dx), A);
      // A(i, i+1) =     D/(dx*dx)
      lis_matrix_set_value(LIS_INS_VALUE, i, i+1,     D/(dx*dx), A);
    } else if (i == ndiv - 2) {
      // A(i, i-1) =     D/(dx*dx)
      lis_matrix_set_value(LIS_INS_VALUE, i, i-1,     D/(dx*dx), A);
      // A(i, i  ) = -2.*D/(dx*dx)
      lis_matrix_set_value(LIS_INS_VALUE, i, i  , -2.*D/(dx*dx), A);
    } else {
      // A(i, i-1) =     D/(dx*dx)
      lis_matrix_set_value(LIS_INS_VALUE, i, i-1,     D/(dx*dx), A);
      // A(i, i  ) = -2.*D/(dx*dx)
      lis_matrix_set_value(LIS_INS_VALUE, i, i  , -2.*D/(dx*dx), A);
      // A(i, i+1) =     D/(dx*dx)
      lis_matrix_set_value(LIS_INS_VALUE, i, i+1,     D/(dx*dx), A);
    }
    // RHS
    if (i == 0) {
      // b[i] = -Tw*D/(dx*dx)
      lis_vector_set_value(LIS_INS_VALUE, i, -Tw*D/(dx*dx), b);
    } else if (i == ndiv - 2) {
      // b[i] = -Te*D/(dx*dx)
      lis_vector_set_value(LIS_INS_VALUE, i, -Te*D/(dx*dx), b);
    } else {
      // b[i] = 0
      lis_vector_set_value(LIS_INS_VALUE, i, 0, b);
    }
  }

  lis_matrix_set_type(A, LIS_MATRIX_CSR);
  lis_matrix_assemble(A);

  std::cout << "Matrix A was build.  Start examining it." << std::endl;
  lis_solve(A, b, x, solver);

  double T[ndiv - 1];
  lis_vector_get_values(x, 0, ndiv - 1, T);

  for (int i = 0; i < ndiv - 1; ++i) {
    std::cout << T[i] << std::endl;
  }

  lis_finalize();

  return EXIT_SUCCESS;
}
